/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.core.commands;

import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Guillermo Gonzalez <guillo.gonzo@gmail.com>
 *
 */
public class AddCommand extends ResourceChangeCommand {

	private final List<? extends IResource> resources;
	private final BzrWorkspaceRoot root;

	public AddCommand(BzrWorkspaceRoot bzrWorkspaceRoot, List<? extends IResource> resources) {
		this.root = bzrWorkspaceRoot;
		this.resources = resources;
	}

	@Override
    public IResource[] execute(IProgressMonitor monitor) throws BazaarException {
		try {
			List<IResource> addableResources = createAddableResources(monitor, resources);
			root.getClient().add(getFiles(resources));
			return addableResources.toArray(new IResource[0]);
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e);
		}
	}

	private static File[] getFiles(List<? extends IResource> resources) {
		File[] files = new File[resources.size()];
		for (int i = 0; i < resources.size(); i++) {
			files[i] = resources.get(i).getLocation().toFile();
		}
		return files;
	}

	static List<IResource> createAddableResources(IProgressMonitor monitor, List<? extends IResource> resources) throws BazaarException {
		List<IProject> projects = new ArrayList<IProject>();
		for (IResource resource : resources) {
			projects.add(resource.getProject());
		}
		IBzrLocalResource[] bzrResources = BzrWorkspaceRoot.getBzrResourcesFor(projects.toArray(new IResource[0]));
		StatusCommand cmd = new StatusCommand(bzrResources, true);
		cmd.run(monitor);
		IBzrResourceStatus[] result = cmd.getStatus();
		Set<IResource> addable = new HashSet<IResource>();
		for (IBzrResourceStatus status : result) {
			if (status.isUnknown()) {
				addable.add(status.getResource().getIResource());
			}
		}
		for (IResource resource : resources) {
			addable.add(resource);
		}
		return new ArrayList<IResource>(addable);
	}
}
