/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.core.commands;

import java.util.regex.Pattern;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.commandline.syntax.IMergeOptions;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.client.core.BranchLocation;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.listeners.BzrChangeStateEvent;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.utils.BzrStatus;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Guillermo Gonzalez <guillo.gonzo AT gmail DOT com>
 *
 */
public class MergeCommand extends MetaModifyCommand {

	private final IBzrBranch branch;
	private final BranchLocation location;
	private Option[] options;
	private BazaarException mergeConflictsException;
	private static final Pattern CONFLICT_PATTERN = Pattern.compile("conflict.*encountered");
	
	public MergeCommand(IBzrBranch branch, BranchLocation location, Option...options) {
		super();
		this.branch = branch;
		this.location = location;
		this.options = options;
	}

	@Override
	public IResource[] execute(IProgressMonitor monitor) throws BazaarException {
		try {
			monitor.beginTask(CoreTexts.MergeCommand_taskName, 100);
			monitor.setTaskName(CoreTexts.MergeCommand_taskName);
			IBazaarClient client = getClient(branch);
			monitor.worked(10);
            client.merge(location, options);
		} catch (BazaarClientException e) {
			if (e.getMessage() != null && CONFLICT_PATTERN.matcher(e.getMessage()).find()) {
				mergeConflictsException = new BazaarException(BzrStatus.INFO, IStatus.OK, CoreTexts.MergeConflicts_error, e);
			} else {
				throw BazaarException.wrapException(e);
			}
		} finally {
			monitor.done();
		}
		return BzrWorkspaceRoot.findProjectsInBranch(branch);
	}

	@Override
	protected BazaarException getPostCommandError() {
		return mergeConflictsException;
	}

	@Override
	protected int getChangeType() {
		for (Option option : options) {
			if(option.equals(IMergeOptions.PULL)) {
				return BzrChangeStateEvent.BRANCH_STATE_MODIFY;
			}
		}
		return BzrChangeStateEvent.WT_STATE_MODIFY;
	}
}
