/**
 *
 */
package org.vcs.bazaar.eclipse.core.commands;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.core.utils.ResourceUtil;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Guillermo Gonzalez <guillo.gonzo@gmail.com>
 *
 */
public abstract class MetaModifyCommand extends BazaarCommand {

	@Override
    public void run(IProgressMonitor monitor) throws BazaarException {
		monitor.beginTask("", 100);
		IProgressMonitor subMonitor = new SubProgressMonitor(monitor, 90);
		final IResource[] resources = execute(subMonitor);
		subMonitor = new SubProgressMonitor(monitor, 10);
		try {
			ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
				public void run(IProgressMonitor monitor) throws CoreException {
					try {
						monitor.beginTask("", resources.length);
						monitor.setTaskName(CoreTexts.MetaModifyCommad_refreshing);
						for (IResource resource : resources) {
							if(resource.exists()) {
								if (monitor.isCanceled()) {
									break;
								}
								resource.refreshLocal(IResource.DEPTH_INFINITE, new SubProgressMonitor(monitor, 1));
								ResourceUtil.markAsDirty(resource);
							}
							monitor.worked(1);
						}
					} finally {
						monitor.done();
					}
				}
			}, subMonitor);
			List<IResource> resourceToRefresh = Arrays.asList(resources);
			EclipseBazaarCore.getDefault().getStatusCacheManager().scheduleRefresh(resourceToRefresh, new NullProgressMonitor(), getChangeType());
		} catch (CoreException e) {
			throw BazaarException.wrapException(e);
		} finally {
			monitor.done();
		}
		if (getPostCommandError() != null) {
			throw getPostCommandError();
		}
	}

	public abstract IResource[] execute(IProgressMonitor monitor) throws BazaarException;

	protected abstract int getChangeType();

	protected BazaarException getPostCommandError() {
		return null;
	}
	
	// helpers
	protected static IResource[] getIResources(IBzrLocalResource[] bzrResources) {
		List<IResource> resources = new ArrayList<IResource>(bzrResources.length);
		for (IBzrLocalResource resource : bzrResources) {
			resources.add(resource.getIResource());
		}
		return resources.toArray(new IResource[0]);
	}

	protected static IResource[] getIResources(List<IBzrLocalResource> bzrResources) {
		return getIResources(bzrResources.toArray(new IBzrLocalResource[0]));
	}

}
