/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.core.commands;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.commandline.syntax.IPullOptions;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.client.core.BranchLocation;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Guillermo Gonzalez
 *
 */
public class PullCommand extends StateChangeCommand {

	private IBzrBranch branch;
	private BranchLocation location;
	private boolean remember = false;

	public PullCommand(IBzrBranch branch, BranchLocation location) {
		this.branch = branch;
		this.location = location;
	}

	@Override
	public IResource[] execute(IProgressMonitor monitor) throws BazaarException {
		try {
			monitor.beginTask(CoreTexts.PullCommand_taskName, 100);
			IBazaarClient client = getClient(branch);
			monitor.worked(10);
			if(remember)
				client.pull(location, IPullOptions.REMEMBER);
			else
				client.pull(location);
			monitor.worked(90);
			return BzrWorkspaceRoot.findProjectsInBranch(branch);
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e);
		} finally {
			monitor.done();
		}
	}

	public void setRemember(boolean remember) {
		this.remember = remember;
	}

}
