/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.core.commands;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.client.core.BranchLocation;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Guillermo Gonzalez
 * TODO: should check here if the project is lightweight/heavyweight checkout?
 */
public class SwitchCommand extends StateChangeCommand {

	private IBzrBranch branch;
	private BranchLocation location;

	public SwitchCommand(IBzrBranch branch, BranchLocation location) {
		this.branch = branch;
		this.location = location;
	}

	@Override
	public IResource[] execute(IProgressMonitor monitor) throws BazaarException {
		try {
			monitor.beginTask("Switch command", 1000);
			monitor.setTaskName("Switching");
			IBazaarClient client = getClient(branch);
			client.switchBranch(location);
			monitor.worked(1000);
			return BzrWorkspaceRoot.findProjectsInBranch(branch);
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e);
		} finally {
			monitor.done();
		}
	}

}
