/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.core.status;

import java.util.Map;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.ITracingOptions;
import org.vcs.bazaar.eclipse.core.commands.StatusUpdaterCommand;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;

/**
 * When the status of a resource is asked, we don't update a resource at once.<br>
 * Based on StatusUpdateStrategy from Subclipse<br>
 *
 * @author Guillermo Gonzalez
 */
public class StatusUpdater {

	public StatusUpdater() {
		super();
	}

	/**
	 * update the status of the given resource or it's parent (if the resource
	 * is a File)
	 *
	 * @param resource
	 * @param monitor
	 * @return
	 * @throws CoreException
	 */
	protected Map<IResource, IBzrResourceStatus> statusesToUpdate(IContainer resource, IProgressMonitor monitor, boolean shallow) throws CoreException {
		EclipseBazaarCore.trace(ITracingOptions.STATUS_COMMAND, "StatusUpdater.statusesToUpdate: " + resource.getProject().getName()+ "/" +resource.getName());
		Map<IResource, IBzrResourceStatus> statuses = null;
		StatusUpdaterCommand cmd = new StatusUpdaterCommand(BzrWorkspaceRoot.getBzrResourceFor(resource), shallow);
		cmd.run(monitor);
		statuses = cmd.getStatus();
		return statuses;
	}

}
