/*
 * OpenBazaarRevisionAction
 *
 * Open an "old" revision in an editor from like "History" view.
 *
 */
package org.vcs.bazaar.eclipse.ui.actions;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFileState;
import org.eclipse.core.resources.IStorage;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.PlatformObject;
import org.eclipse.core.runtime.content.IContentType;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.team.core.history.IFileRevision;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IEditorRegistry;
import org.eclipse.ui.IPersistableElement;
import org.eclipse.ui.IStorageEditorInput;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchSite;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.BaseSelectionListenerAction;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.model.IWorkbenchAdapter;
import org.eclipse.ui.progress.IProgressService;

public class OpenBazaarRevisionAction extends BaseSelectionListenerAction {

	public class BazaarRevisionEditorInput extends PlatformObject implements IWorkbenchAdapter, IStorageEditorInput {

		private IFileRevision fileRevision;
		private IStorage storage;

		public BazaarRevisionEditorInput(IFileRevision revision) {
			this.fileRevision = revision;
			try {
				this.storage = revision.getStorage(new NullProgressMonitor());
			} catch (CoreException e) {
			}
		}

		public Object[] getChildren(Object o) {
			System.out.println("OpenBazaarRevisionAction::BazaarRevisionEditorInput::getChildren()");
			return new Object[0];
		}

		public ImageDescriptor getImageDescriptor(Object object) {
			System.out.println("OpenBazaarRevisionAction::BazaarRevisionEditorInput::getImageDescriptor()");
			return null;
		}

		public String getLabel(Object o) {
			return getName();
/*			if (storage != null) {
				return storage.getName();
			}
			return ""; //$NON-NLS-1$
			*/
		}

		public Object getParent(Object o) {
			System.out.println("OpenBazaarRevisionAction::BazaarRevisionEditorInput::getParent()");
			return null;
		}

		public IStorage getStorage() throws CoreException {
			return storage;
		}

		public boolean exists() {
			System.out.println("OpenBazaarRevisionAction::BazaarRevisionEditorInput::exists()");
			return true;
		}

		public ImageDescriptor getImageDescriptor() {
			System.out.println("OpenBazaarRevisionAction::BazaarRevisionEditorInput::getImageDescriptor()");
			return null;
		}

		public String getName() {
			return fileRevision.getName() + " " + fileRevision.getContentIdentifier(); //$NON-NLS-1$
		}

		public IPersistableElement getPersistable() {
			return null; // Can to save editor changes
		}

		public String getToolTipText() {
			return getName();
		}

        @SuppressWarnings("unchecked")
        @Override
        public Object getAdapter(Class adapter) {
			if (adapter == IWorkbenchAdapter.class) {
				return this;
			}
			if (adapter == IFileRevision.class) {
				return fileRevision;
			} else if (adapter == IFileState.class) {
				if (storage != null && storage instanceof IFileState) {
					return storage;
				}
			}
			return super.getAdapter(adapter);
		}

	}

	private IStructuredSelection selection;
	private final IWorkbenchSite site;

	public OpenBazaarRevisionAction(String text, IWorkbenchSite site) {
		super(text);
		this.site = site;
	}

	@Override
    public void run() {
		IStructuredSelection structSel = selection;

		Object[] objArray = structSel.toArray();

		for (Object tempRevision : objArray) {
			final IFileRevision revision = (IFileRevision) tempRevision;
			if (revision == null || !revision.exists()) {
				MessageDialog.openError(site.getShell(), "Deleted Revision", "Can't open a deleted revision");
			} else {
				IRunnableWithProgress runnable = new IRunnableWithProgress() {
					public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
						IStorage file;
						try {
							file = revision.getStorage(monitor);
							String id = getEditorID(file.getName(), file.getContents());

							if (file instanceof IFile) {
								// if this is the current workspace file, open
								// it
								IDE.openEditor(site.getPage(), (IFile) file);
							} else if (showEditor(revision)) {
								BazaarRevisionEditorInput fileRevEditorInput = new BazaarRevisionEditorInput(revision);
								if (!editorAlreadyOpenOnContents(fileRevEditorInput)) {
									site.getPage().openEditor(fileRevEditorInput, id);
								}
							}
						} catch (CoreException e) {
							throw new InvocationTargetException(e);
						}

					}

					private boolean showEditor(IFileRevision revision) {
						return true;
					}
				};
				IProgressService progressService = PlatformUI.getWorkbench().getProgressService();
				try {
					progressService.run(false, false, runnable);
				} catch (InvocationTargetException e) {
				} catch (InterruptedException e) {
				}
			}

		}
	}

	String getEditorID(String fileName, InputStream contents) {
		IWorkbench workbench = PlatformUI.getWorkbench();
		IEditorRegistry registry = workbench.getEditorRegistry();
		IContentType type = null;
		if (contents != null) {
			try {
				type = Platform.getContentTypeManager().findContentTypeFor(contents, fileName);
			} catch (IOException e) {
			}
		}
		if (type == null) {
			type = Platform.getContentTypeManager().findContentTypeFor(fileName);
		}
		IEditorDescriptor descriptor = registry.getDefaultEditor(fileName, type);
		String id;
		if (descriptor == null || descriptor.isOpenExternal()) {
			id = "org.eclipse.ui.DefaultTextEditor"; //$NON-NLS-1$
		} else {
			id = descriptor.getId();
		}

		return id;
	}

	@Override
    protected boolean updateSelection(IStructuredSelection selection) {
		this.selection = selection;
		return shouldShow();
	}

	protected boolean shouldShow() {
		Object[] objArray = selection.toArray();

		if (objArray.length == 0) {
            return false;
        }

		for (Object element : objArray) {
			IFileRevision revision = (IFileRevision) element;
			// check to see if any of the selected revisions are deleted
			// revisions
			if (revision != null && !revision.exists()) {
                return false;
            }
		}

		return true;
	}

	private boolean editorAlreadyOpenOnContents(BazaarRevisionEditorInput input) {
		IEditorReference[] editorRefs = site.getPage().getEditorReferences();
		for (IEditorReference editorRef : editorRefs) {
			IEditorPart part = editorRef.getEditor(false);
			if (part != null && part.getEditorInput() instanceof BazaarRevisionEditorInput) {
				IFileRevision inputRevision = (IFileRevision) input.getAdapter(IFileRevision.class);
				IFileRevision editorRevision = (IFileRevision) part.getEditorInput().getAdapter(IFileRevision.class);

				if (inputRevision.equals(editorRevision)) {
					// make the editor that already contains the revision
					// current
					site.getPage().activate(part);
					return true;
				}
			}
		}
		return false;
	}

}
