/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.Collection;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.client.core.BranchLocation;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.commands.RebaseCommand;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalBranch;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.dialogs.MergeDialog;
import org.vcs.bazaar.eclipse.ui.dialogs.RebaseDialog;
import org.vcs.bazaar.eclipse.ui.dialogs.RebaseDialog.RebaseDialogInfo;

/**
 * @author Piotr Piastucki
 */
public class RebaseAction extends WorkbenchAction {

	@Override
	protected void execute(IAction action) throws BazaarException, InvocationTargetException, InterruptedException {
		final IProject[] projects = getEnclosingProjectsForSelectedResources();
		final IBzrBranch branch = BzrWorkspaceRoot.getBranchFor(projects[0]);
		final RebaseDialogInfo dialogInfo = new RebaseDialogInfo();
		final BranchLocation remoteLocation = getRemoteLocation(branch, dialogInfo);
		if (remoteLocation != null) {
			run(new WorkspaceModifyOperation() {
				@Override
                public void execute(IProgressMonitor monitor) throws InvocationTargetException {
					try {
						monitor.beginTask("", 1000);
						monitor.setTaskName(CoreTexts.RebaseCommand_taskName);
						RebaseCommand cmd = new RebaseCommand(branch, remoteLocation, dialogInfo.getOptions().toArray(new Option[0]));
						cmd.run(EclipseBazaarCore.subProgressMonitorFor(monitor));
					} catch (BazaarException e) {
						EclipseBazaarUI.log(e.getMessage(), e);
						throw new InvocationTargetException(e, e.getMessage());
					} finally {
						monitor.done();
					}
				}
			}, false /* no cancelable */, PROGRESS_DIALOG);
		}
	}

	private BranchLocation getRemoteLocation(IBzrBranch branch, RebaseDialogInfo dialogInfo) throws BazaarException {
		Collection<IBzrBranch> locations = getAllBranchLocations(branch);
		IBzrBranch initialBranch = null;
		if (branch instanceof IBzrLocalBranch) {
			initialBranch = ((IBzrLocalBranch) branch).getParent();
		}
		RebaseDialog dialog = new RebaseDialog(getShell(), locations, initialBranch, dialogInfo);
		dialog.setBlockOnOpen(true);
		if (dialog.open() == MergeDialog.OK) {
            try {
				return new BranchLocation(dialogInfo.getLocation());
			} catch (BazaarClientException e) {
			}
		}
		return null;
	}

	@Override
	protected boolean isEnabledForMultipleEclosingProjects() {
		return isSameBranchInEnclosingProjects();
	}
	
	@Override
	protected boolean isEnabledByType(IResource resource) {
		return resource.getType() == IResource.PROJECT && EclipseBazaarCore.getDefault().getPlugin("rewrite") != null;
	}

}
