/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.ui.actions;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.core.commands.UpdateCommand;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 *
 * @author Guillermo Gonzalez
 *
 */
public class UpdateAction extends WorkbenchAction {

	/**
	 * <p>
	 * The action has been activated. The argument of the method represents the
	 * 'real' action sitting in the workbench UI.
	 * </p>
	 *
	 * @throws BazaarException
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	@Override
	protected void execute(IAction action) throws BazaarException, InvocationTargetException, InterruptedException {
		IProject project = getSelectedProjects()[0];
		final UpdateCommand cmd = new UpdateCommand(BzrWorkspaceRoot.getBranchFor(project));
        String updMsg = null;
        try {
            run(new WorkspaceModifyOperation() {
                @Override
                public void execute(IProgressMonitor monitor)
                    throws InvocationTargetException, BazaarException
                {
                    try {
                        cmd.run(monitor);
                    } finally {
                        monitor.done();
                    }
                }
            }, false /* no cancelable */, PROGRESS_DIALOG);
        } catch (InvocationTargetException ite) {
            Throwable ie = ite.getCause();
            if (ie instanceof BazaarException) {
                if (ie.getCause() != null) {
                    ie = ie.getCause();
                }
                if (ie.getMessage().contains("conflicts encountered")) {
                    updMsg = "Conflicts occurred during update.\n\n" + ie.getMessage();
                    ite = null;
                }
            }
            if (ite != null) {
                throw ite;
            }
        }
        try {
            project.refreshLocal(IResource.DEPTH_INFINITE, null);
        } catch (CoreException e) {
            throw new BazaarException(e);
        }
        if (updMsg != null) {
            MessageDialog.openInformation(getShell(), "Update finished", updMsg);
        }
	}

	@Override
	protected boolean isEnabledByType(IResource resource) {
		if (resource.getType() == IResource.PROJECT) {
			BzrWorkspaceRoot root = BzrWorkspaceRoot.getBzrWorkspaceRootFor((IProject)resource);
			return root.getBranch().isBound();
		}
		return false;
	}

	@Override
	protected boolean isEnabledForMultipleEclosingProjects() {
		return isSameBranchInEnclosingProjects();
	}

}
