/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.ui.compare;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.compare.CompareConfiguration;
import org.eclipse.compare.IEditableContent;
import org.eclipse.compare.ITypedElement;
import org.eclipse.compare.ResourceNode;
import org.eclipse.compare.structuremergeviewer.DiffNode;
import org.eclipse.compare.structuremergeviewer.ICompareInput;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.ui.synchronize.SaveableCompareEditorInput;
import org.eclipse.ui.IWorkbenchPage;

/**
 * Compare editor input that is able to save the left side of the compare
 *
 * @author Guillermo Gonzalez <guillo.gonzo AT gmail DOT com>
 *
 */
public class SaveableBazaarCompareEditorInput extends SaveableCompareEditorInput {

	private static class MyDiffNode extends DiffNode {

		private String name;
		
		public MyDiffNode(int kind, ITypedElement left, ITypedElement right) {
			super(kind, null, left, right);
		}

		public MyDiffNode(ITypedElement left, ResourceNode right) {
			super(left, right);
		}

		@Override
		protected void fireChange() {
			super.fireChange();
		}
		
		@Override
		public String getName() {
			return name;
		}
		
		public void setName(String name) {
			this.name = name;
		}
	}

	private BazaarResourceNode left;
	private ResourceNode right;

	/**
	 * This constructor use a default configuration with both side read-only
	 * @param resource
	 * @param left
	 * @param right
	 */
	public SaveableBazaarCompareEditorInput(IResource resource, BazaarResourceNode left, ResourceNode right, IWorkbenchPage page) {
		this(new CompareConfiguration(), resource, left, right, page);
	}

	public SaveableBazaarCompareEditorInput(CompareConfiguration configuration,
			IResource resource, BazaarResourceNode left, ResourceNode right, IWorkbenchPage page) {
		super(configuration, page);
		this.left = left;
		this.right = right;
	}

	@Override
	protected void fireInputChange() {
		((MyDiffNode)getCompareInput()).fireChange();
	}

	@Override
	protected ICompareInput prepareCompareInput(IProgressMonitor monitor)
			throws InvocationTargetException, InterruptedException {
		ITypedElement left = createFileElement((IFile)this.left.getResource());
		MyDiffNode input = new MyDiffNode(left, right);
		getCompareConfiguration().setLeftEditable(isLeftEditable(input));
		getCompareConfiguration().setRightEditable(false);
		initLabels(input);
		input.setName(getTitle());
		return input;

	}

	private void initLabels(ICompareInput input) {
		CompareEditorUtil.setLabels(this, input.getLeft(), input.getRight(), null, null);
	}

	private boolean isLeftEditable(ICompareInput input) {
		Object left = input.getLeft();
		if (left instanceof IEditableContent) {
			return ((IEditableContent) left).isEditable();
		}
		return false;
	}

}
