/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.dialogs;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.bindings.keys.KeyStroke;
import org.eclipse.jface.fieldassist.ContentProposal;
import org.eclipse.jface.fieldassist.ContentProposalAdapter;
import org.eclipse.jface.fieldassist.IContentProposal;
import org.eclipse.jface.fieldassist.IContentProposalProvider;
import org.eclipse.jface.fieldassist.IControlContentAdapter;
import org.eclipse.jface.fieldassist.IControlContentAdapter2;
import org.eclipse.jface.fieldassist.TextContentAdapter;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Text;

/**
 * @author Piotr Piastucki
 */
public class CommitMessageArea {

	public static Text getCommitMessageArea(Composite parent, int style) {
		Text messageArea = new Text(parent, style);
		setupContentAssist(messageArea);
		return messageArea;
	}

	protected static void setupContentAssist(Control control) {
		try {
			KeyStroke keyStroke = KeyStroke.getInstance("Ctrl+Space");
			CommitMessageContentProposalProvider proposalProvider = new CommitMessageContentProposalProvider();
			ContentProposalAdapter contentProposalAdapter = new ContentProposalAdapter(control,
					new TextContentAdapter(), proposalProvider, keyStroke, null);
			contentProposalAdapter.setProposalAcceptanceStyle(ContentProposalAdapter.PROPOSAL_INSERT);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private static class CommitMessageContentProposalProvider implements IContentProposalProvider {

		private List<String> proposals;
		private IContentProposal[] contentProposals;
		private boolean filterProposals = false;

		public CommitMessageContentProposalProvider() {
			this.proposals = new ArrayList<String>(CommitMessageHistory.getCommitHistory());
		}

		public IContentProposal[] getProposals(String contents, int position) {
			if (contentProposals == null) {
				contentProposals = new IContentProposal[proposals.size()];
				int i = 0;
				for (String proposal : proposals) {
					String label = proposal;
					int nlIndex = label.indexOf('\n');
					if (nlIndex > 0) {
						label = label.substring(0, nlIndex);
					}
					contentProposals[i++] = new ContentProposal(proposal, label, proposal);
				}
			}
			if (filterProposals) {
				ArrayList<IContentProposal> list = new ArrayList<IContentProposal>();
				for (IContentProposal proposal : contentProposals) {
					if (proposal.getLabel().length() >= contents.length()
							&& proposal.getLabel().substring(0, contents.length()).equalsIgnoreCase(contents)) {
						list.add(proposal);
					}
				}
				return list.toArray(new IContentProposal[list.size()]);
			}
			return contentProposals;
		}

	}

	public static class StyledTextContentAdapter implements IControlContentAdapter, IControlContentAdapter2 {

		public Point getSelection(Control control) {
			return ((StyledText) control).getSelection();
		}

		public void setSelection(Control control, Point range) {
			((StyledText) control).setSelection(range);
		}

		public void setControlContents(Control control, String contents, int cursorPosition) {
			((StyledText) control).setText(contents);
			((StyledText) control).setSelection(cursorPosition, cursorPosition);
		}

		public void insertControlContents(Control control, String contents, int cursorPosition) {
			Point selection = ((StyledText) control).getSelection();
			((StyledText) control).insert(contents);
			if (cursorPosition < contents.length()) {
				((StyledText) control).setSelection(selection.x + cursorPosition, selection.x + cursorPosition);
			}
		}

		public String getControlContents(Control control) {
			return ((StyledText) control).getText();
		}

		public int getCursorPosition(Control control) {
			return ((StyledText) control).getCaretOffset();
		}

		public Rectangle getInsertionBounds(Control control) {
			StyledText text = (StyledText) control;
			Point caretOrigin = text.getLocationAtOffset(text.getCaretOffset());
			return new Rectangle(caretOrigin.x + text.getClientArea().x, caretOrigin.y + text.getClientArea().y + 3, 1,
					text.getLineHeight());
		}

		public void setCursorPosition(Control control, int index) {
			((StyledText) control).setCaretOffset(index);
		}

	}
}
