/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.operations;

import java.lang.reflect.InvocationTargetException;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableContext;
import org.vcs.bazaar.client.IBazaarRevisionSpec;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.eclipse.core.commands.CreatePatchCommand;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Piotr Piastucki
 */
public class CreatePatchOperation extends BzrOperation {

	private final List<IBzrLocalResource> resources;
	private final IBazaarRevisionSpec revision;
	private final Option[] options;
	private String patchContent;

	public CreatePatchOperation(IRunnableContext context, List<IBzrLocalResource> resources, IBazaarRevisionSpec revision, Option[] options) {
		super(context);
		this.resources = resources;
		this.revision = revision;
		this.options = options;
	}
	
	public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
		CreatePatchCommand cmd = new CreatePatchCommand(resources, revision, options);
		try {
			cmd.run(monitor);
			patchContent = cmd.getPatchContent();
		} catch (BazaarException e) {
			throw new InvocationTargetException(e);
		}
	}

	public String getPatchContent() {
		return patchContent;
	}

}
