/**
 *
 */
package org.vcs.bazaar.eclipse.ui.operations;

import java.lang.reflect.InvocationTargetException;
import java.util.List;

import org.eclipse.compare.CompareEditorInput;
import org.eclipse.compare.CompareUI;
import org.eclipse.compare.ResourceNode;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableContext;
import org.eclipse.team.core.TeamException;
import org.eclipse.ui.IWorkbenchPage;
import org.vcs.bazaar.client.BazaarRevision;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.resources.IStorageBazaarRevision;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.compare.BazaarCompareEditorInput;
import org.vcs.bazaar.eclipse.ui.compare.BazaarResourceNode;
import org.vcs.bazaar.eclipse.ui.compare.BazaarRevisionNode;
import org.vcs.bazaar.eclipse.ui.compare.BazaarStatusBasedCompareEditorInput;
import org.vcs.bazaar.eclipse.ui.compare.SaveableBazaarCompareEditorInput;

/**
 * This class don't implement it run method, it only provides static methods
 * to open a compare editor or dialog
 *
 * @author Guillermo Gonzalez <guillo.gonzo AT gmail DOT com>
 *
 */
public class DiffOperation extends BzrOperation {

	protected DiffOperation(IRunnableContext context) {
		super(context);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.operation.IRunnableWithProgress#run(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void run(IProgressMonitor monitor) throws InvocationTargetException,
			InterruptedException {
	}

	/**
	 * Open a SaveableCompareEditor showing the comparison of local vs last revision
	 * @param file
	 * @throws TeamException
	 */
	public static void openSaveableEditor(IResource file, IWorkbenchPage page) {
		openSaveableEditor(file, new IStorageBazaarRevision(file), page);
	}

	/**
	 * Open a SaveableCompareEditor showing the comparison of local vs selected revision
	 * @param file
	 * @throws TeamException
	 */
	public static void openSaveableEditor(IResource file, BazaarRevision revision) throws TeamException {
		openSaveableEditor(file, new IStorageBazaarRevision(file, revision), EclipseBazaarUI.getActivePage());
	}

	/**
	 * Open a SaveableCompareEditor showing the comparison of selected revision  vs another selected revision
	 * @param file
	 * @throws TeamException
	 */
	public static void openEditor(IStorageBazaarRevision left, IStorageBazaarRevision right) {
		CompareEditorInput editorInput = getEditorInput(getNode(left), getNode(right));
		CompareUI.openCompareEditor(editorInput);
	}

	/**
	 * Open a CompareDialog showing the comparison of local vs last revision contents
	 * @param file
	 */
	public static void openDialog(IResource file) {
		CompareEditorInput editorInput = getEditorInput(new BazaarResourceNode(file), getNode(new IStorageBazaarRevision(file)));
		CompareUI.openCompareDialog(editorInput);
	}

	public static void openSaveableEditor(IResource left, IStorageBazaarRevision right, IWorkbenchPage page) {
		CompareEditorInput editorInput = getSaveableEditorInput(left, right, page);
		CompareUI.openCompareEditorOnPage(editorInput, page);
	}

	/**
	 * @param left {@link ResourceNode}
	 * @param right {@link ResourceNode}
	 * @param resource {@link IResource}
	 * @return
	 */
	private static CompareEditorInput getEditorInput(ResourceNode left, ResourceNode right) {
		CompareEditorInput editorInput = new BazaarCompareEditorInput(left, right);
		return editorInput;
	}

	/**
	 * @param left {@link IResource}
	 * @param right {@link ResourceNode}
	 * @param page
	 * @return
	 */
	private static CompareEditorInput getSaveableEditorInput(IResource left, IStorageBazaarRevision right, IWorkbenchPage page) {
		CompareEditorInput editorInput = new SaveableBazaarCompareEditorInput(left,
				left != null ? new BazaarResourceNode(left) : getNode(right),
				left != null ? getNode(right) : new ResourceNode(left), page);
		return editorInput;
	}

	private static BazaarRevisionNode getNode(IStorageBazaarRevision rev) {
		  return rev == null ? null : new BazaarRevisionNode(rev);
	}

	public static void openCompareEditor(IResource[] resources, List<IBzrResourceStatus> localResources,
			BazaarRevision leftRevision, BazaarRevision rightRevision, IWorkbenchPage activePage) {
		CompareEditorInput editorInput = new BazaarStatusBasedCompareEditorInput(resources, localResources, leftRevision, rightRevision, activePage);
		CompareUI.openCompareEditorOnPage(editorInput, activePage);
	}

}
