/**
 *
 */
package org.vcs.bazaar.eclipse.ui.operations;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.IWorkbenchPart;
import org.vcs.bazaar.client.core.BranchLocation;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.core.commands.PushCommand;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalBranch;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.internal.core.model.ProjectBranch;

/**
 * @author Guillermo Gonzalez <guillo.gonzo@gmail.com>
 *
 */
public class PushOperation extends BzrOperation {

	private IBzrBranch branch;
	private BranchLocation location;
	private boolean remember;
	private boolean useExsitingDir;

	public PushOperation(IBzrBranch branch, BranchLocation location, boolean remember, boolean useExsitingDir) {
		this(null, branch, location, remember, useExsitingDir);
	}

	public PushOperation(IBzrBranch branch) {
		this(null, branch, getPushLocation(branch), false, false);
	}

	private static BranchLocation getPushLocation(IBzrBranch branch) {
		if (branch instanceof IBzrLocalBranch) {
			return ((IBzrLocalBranch) branch).getPushBranch().getBranchLocation();
		}
		return null;
	}

	public PushOperation(IWorkbenchPart part, IBzrBranch branch, BranchLocation location, boolean remember, boolean useExsitingDir) {
		super(part);
		this.branch = branch;
		this.location = location;
		this.remember = remember;
		this.useExsitingDir = useExsitingDir;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.operation.IRunnableWithProgress#run(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
		try {
			monitor.beginTask(CoreTexts.bind(CoreTexts.PushCommand_taskName, location.toString()), 1000);
			PushCommand cmd = new PushCommand(branch, location);
			cmd.setRemember(remember);
			cmd.setUseExsitingDir(useExsitingDir);
			cmd.run(monitor);
			if (branch instanceof ProjectBranch) {
				((ProjectBranch)branch).setDirty(true);
			}
		} catch (BazaarException e) {
			throw new InvocationTargetException(e);
		} finally {
			monitor.done();
		}
	}

	@Override
	protected String getJobName() {
		return "Pushing " + branch.getName();
	}

	@Override
	protected boolean canRunAsJob() {
		return true;
	}

}
