/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.widgets;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.vcs.bazaar.client.BazaarStatusKind;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.ui.UITexts;

/**
 * @author Piotr Piastucki
 */
public class BzrResourceSelectionPathLabelProvider extends ColumnLabelProvider {

	private static final Map<BazaarStatusKind, String> FILE_STATUS_DESCRIPTION = new HashMap<BazaarStatusKind, String>(6);
	static {
		FILE_STATUS_DESCRIPTION.put(BazaarStatusKind.CREATED, UITexts.CommitDialog_statusAdded);
		FILE_STATUS_DESCRIPTION.put(BazaarStatusKind.MODIFIED, UITexts.CommitDialog_statusModified);
		FILE_STATUS_DESCRIPTION.put(BazaarStatusKind.DELETED, UITexts.CommitDialog_statusRemoved);
		FILE_STATUS_DESCRIPTION.put(BazaarStatusKind.RENAMED, UITexts.CommitDialog_statusRenamed);
		FILE_STATUS_DESCRIPTION.put(BazaarStatusKind.UNKNOWN, UITexts.CommitDialog_statusUntracked);
	}
	
	@Override
	public String getText(Object obj) {
		return ((IBzrResourceStatus) obj).getStatus().getPath();
	}

	@Override
	public String getToolTipText(Object element) {
		IBzrResourceStatus resource = (IBzrResourceStatus) element;
		if (resource.contains(BazaarStatusKind.RENAMED)) {
			return FILE_STATUS_DESCRIPTION.get(BazaarStatusKind.RENAMED);
		}
		if (resource.contains(BazaarStatusKind.CREATED)) {
			return FILE_STATUS_DESCRIPTION.get(BazaarStatusKind.CREATED);
		}
		if (resource.contains(BazaarStatusKind.MODIFIED)) {
			return FILE_STATUS_DESCRIPTION.get(BazaarStatusKind.MODIFIED);
		}
		if (resource.contains(BazaarStatusKind.DELETED)) {
			return FILE_STATUS_DESCRIPTION.get(BazaarStatusKind.DELETED);
		}
		if (resource.contains(BazaarStatusKind.UNKNOWN)) {
			return FILE_STATUS_DESCRIPTION.get(BazaarStatusKind.UNKNOWN);
		}
		return "";
	}
}