/*******************************************************************************
 * Copyright (c) 2012 Idereal Limited
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Taler
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.wizards.createproject;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Scrollable;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.operations.CreateBazaarProjectOperation;

/**
 * A specialization of CreateProjectEntry.
 * 
 * Provides a multi selection box to choose the branch type, Branch, Checkout or
 * Lightweight Checkout.
 * 
 * @author Alexander Taler
 */
public class CreateProjectBranchType extends CreateProjectEntry {

    private static String labelText = "Branch Type";

    protected Button coButton;
    protected Button lwcoButton;
    protected Button brButton;

    public CreateProjectBranchType(Composite parent,
                                   WizardPage page,
                                   CreateProjectWizard wiz,
                                   int width) {
        super(parent, page, wiz, labelText, width, "");
    }

    @Override
    protected Scrollable createWidget(Composite parent) {
        Composite compo = new Composite(parent, SWT.NONE);
        compo.setLayout(new GridLayout(1, false));

        coButton = new Button(compo, SWT.RADIO);
        coButton.setText("Checkout");
        coButton.setSelection(true);

        lwcoButton = new Button(compo, SWT.RADIO);
        lwcoButton.setText("Lightweight Checkout");

        brButton = new Button(compo, SWT.RADIO);
        brButton.setText("Branch");

        return compo;
    }

    @Override
    public void isValid() throws BazaarException {
        return;
    }

    @Override
    public String getValue() {
        return "";
    }

    public int getBranchType() {
        if (coButton.getSelection()) {
            return CreateBazaarProjectOperation.TYPE_CHECKOUT;
        }
        if (lwcoButton.getSelection()) {
            return CreateBazaarProjectOperation.TYPE_LIGHTWEIGHT;
        }
        if (brButton.getSelection()) {
            return CreateBazaarProjectOperation.TYPE_BRANCH;
        }
        throw new RuntimeException("No Branch Type selected");
    }

    @Override
    public void setValue(String newValue) {
        throw new RuntimeException("setValue not supported");
    }
}
