/*******************************************************************************
 * Copyright (c) 2012 Idereal Limited
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Taler
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.wizards.createproject;

import org.eclipse.core.runtime.Platform;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.vcs.bazaar.eclipse.ui.wizards.BzrWizardPage;

/**
 * The basic page for the Bazaar project creation Wizard.
 *
 * The page is has these entries, in order:
 *  - Remote Bazaar URL
 *  - Project Name
 *  - Local directory location
 *
 * @author Alexander Taler alex <@> idereal <.> co <.> nz
 */
public class CreateProjectPage extends BzrWizardPage {

    // The parent wizard to act as a reference
    public CreateProjectWizard wiz;

    CreateProjectRemoteBranch remoteBranch = null;
    CreateProjectName projName = null;
    CreateProjectLocation projLoc = null;

    public CreateProjectPage(String pageName, CreateProjectWizard wiz) {
        super(pageName);
        this.wiz = wiz;
        setTitle("Eclipse Project from Bazaar");
        setDescription("Create a local Eclipse project connected to a remote Bazaar branch");
	}

	public void createControl(Composite parent) {
        final Composite topComposite = new Composite(parent, SWT.NONE);
        topComposite.setLayout(new GridLayout(1, false));

        ///////////////
        // Repository Location
        remoteBranch = new CreateProjectRemoteBranch(topComposite, this, wiz, 425, "");
        setRepoLocation(wiz.getInitialBranch());

        // Space
        Label space1Label = new Label(topComposite, SWT.LEFT);
        space1Label.setText("");

        ///////////////
        // Project Name
        projName = new CreateProjectName(topComposite, this, wiz, "Name of Eclipse Project", 150, "");

        // Space
        Label space2Label = new Label(topComposite, SWT.LEFT);
        space2Label.setText("");

        ///////////////
        // Project Location
        String bv = Platform.getLocation().addTrailingSeparator().toOSString();

        projLoc = new CreateProjectLocation(topComposite, this, wiz, "Location for Eclipse Project", 425, bv);

        // Set up the connections
        remoteBranch.setCascadeTo(projName);
        projName.setCascadeTo(projLoc);

        setControl(topComposite);
		setPageComplete(false);
	}

	@Override
	public boolean isPageComplete() {
        try {
            remoteBranch.isValid();
            projName.isValid();
            projLoc.isValid();
        } catch (Exception e) {
            // NullPointerException and BazaarException
            return false;
        }
        return true;
	}

    public String getProjectName() {
        return projName.getValue();
    }

    public String getRepoLocation() {
        return remoteBranch.getValue();
    }

    public void setRepoLocation(String newValue) {
        if (newValue != null) {
            remoteBranch.setValue(newValue);
        }
    }

    public String getProjectLocation() {
        return projLoc.getValue();
    }
}
